package controllers

import (
	"errors"
	"fmt"
	"net/http"

	"github.com/gin-gonic/gin"
)

type RespFunc func(c *gin.Context) (interface{}, error)

// RespJSON 反馈给前端的内容
type RespJSON struct {
	Code    int         `json:"code"`
	Result  interface{} `json:"result"`
	Message string      `json:"message"`
}

type RespError struct {
	Code    int
	Message string
}

func (e RespError) Error() string {
	return fmt.Sprintf("[%d] - %s", e.Code, e.Message)
}

func NewError(code int, message string) RespError {
	return RespError{
		Code:    code,
		Message: message,
	}
}

// Response 封装反馈函数
// gin middleware 没办法拿到函数值
func Response(f RespFunc) gin.HandlerFunc {
	return func(c *gin.Context) {
		var data RespJSON
		status := http.StatusOK

		resp, err := f(c)

		// 如果返回错误
		if err != nil {
			var respError *RespError
			if errors.As(err, respError) {
				data = RespJSON{
					Code:    respError.Code,
					Message: respError.Message,
				}

				if data.Code == 0 {
					data.Code = -1
				}
			} else {
				data = RespJSON{
					Code:    -1,
					Message: err.Error(),
				}
			}
			c.JSON(status, data)
			return
		}

		// 如果输出 bool 说明 f 函数中已经处理了返回
		if _, ok := resp.(bool); ok {
			return
		}

		data = RespJSON{
			Code:   0, // 成功
			Result: resp,
		}
		c.JSON(status, data)
	}
}
